!
! Copyright (C) 1991-2003  ; All Rights Reserved ; ATMET, LLC
! 
! This file is free software; you can redistribute it and/or modify it under the
! terms of the GNU General Public License as published by the Free Software 
! Foundation; either version 2 of the License, or (at your option) any later version.
! 
! This software is distributed in the hope that it will be useful, but WITHOUT ANY 
! WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
! PARTICULAR PURPOSE.  See the GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License along with this 
! program; if not, write to the Free Software Foundation, Inc., 
! 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
!======================================================================================

!-----------------------------------------------------------------
!  Program to prepare "generic" observational and gridded pressure
!    data for input to RAMS/ISAN. See additional comments below
!    for structure. This program is set up to handle one data
!    time per execution.
!-----------------------------------------------------------------

program RAMS_data_prep

implicit none

include  'prep_com.h'

character csuff*3,locfn*80,cmon(12)*3
data cmon/'jan','feb','mar','apr','may','jun'  &
     ,'jul','aug','sep','oct','nov','dec'/
integer :: ioffp
real :: spval
common/conre1/ioffp,spval

real :: obslat1,obslat2,obslon1,obslon2

namelist/data_prep/ipress,irawi,isurface  &
     ,ihour,idate,imonth,iyear  &
     ,innpr,inrawi,insrfce,ifnpress,ifnrawi,ifnsfc  &
     ,wplon,splat,nprx,npry,nprz,levpr  &
     ,obslat1,obslat2,obslon1,obslon2

integer :: nc
integer, parameter :: iu=1,iv=2,it=3,iz=4,ir=5

real, allocatable :: p(:,:,:,:)

!-----------------------------------------------------------------
!  Read input namelist for various parameters. These parameters
!     do not have to come from this namelist; you can define
!     them in the code here if you wish. All parameters that
!     exist in the namelist should be set, however. See the
!     file "namelist.info" for a description of the parameters.
!-----------------------------------------------------------------

open(1,file='PREP_IN',status='old')
read(1,data_prep)
close(1)

!-----------------------------------------------------------------
!    Allocate an array for the pressure data that you will extract
!       from the input file. This will be a 4-dimensional array that
!       is dimensioned by: p(nprx,npry,nprz,ndat)
!       The first 3 dimensions should correspond to the input variables
!       from your namelist.
!          where:
!                nprx - number of points in east-west direction
!                npry - number of points in north-south direction
!                nprz - number of pressure levels
!                ndat - number that is 5 (5 data arrays)


allocate (p(nprx,npry,nprz,7))


if(ipress == 1) then
!-----------------------------------------------------------------
!    Let's do the gridded pressure data first.
!
!       The 5 data arrays are pointed to by:
!           iu - u-component (m/s)
!           iv - v-component (m/s)
!           it - temerature (K)
!           iz - height (m)
!           ir - relative humidity (fraction)
!
!   Note: Grid point (1,1,1) of any of the data arrays is in the
!         lowest southwest corner of the domain.
!-----------------------------------------------------------------
!-----------------------------------------------------------------
!  Set the x and y grid spacing of the gridded data
!-----------------------------------------------------------------

   spcnprx=2.5
   spcnpry=2.5
!-----------------------------------------------------------------
!  Call a subroutine to access the pressure data on a lat-lon grid.
!    All we need to do here is to fill up the 5 data arrays with the
!    subset of the domain requested in the namelist parameters.
!    A stub of this type of routine is included in the file
!    "press_read.f".
!-----------------------------------------------------------------
   call press_read(nprx,npry,nprz,p(1,1,1,iu),p(1,1,1,iv)  &
        ,p(1,1,1,it),p(1,1,1,iz),p(1,1,1,ir))


!-----------------------------------------------------------------
!  Call a routine to write the pressure file in the "standard"
!    RAMS format. First create the file name with a timestamp,
!    then write the pressure data to it.
!-----------------------------------------------------------------
   nc=len_trim(ifnpress)
   write(locfn,'(a,i4.4,a1,i2.2,a1,i2.2,a1,i4.4)' ) &
             ifnpress(1:nc),iyear,'-',imonth,'-',idate,'-',ihour
   open(10,file=locfn,status='unknown',form='formatted')
   rewind 10
   call pressio(10,locfn,p,nprx,npry,nprz,5)
   close(10)

endif


if( irawi.eq. 1) then
!-----------------------------------------------------------------
!  Now process the rawindsondes. Open a file for the soundings with
!    the timestamp. Then call a routine to read the rawindsondes,
!    do some optional processing, then write them to this file.
!    A sample routine is found in "read_rawi.f".
!-----------------------------------------------------------------
        
   nc=len_trim(ifnrawi)
   write(locfn,'(a,i4.4,a1,i2.2,a1,i2.2,a1,i4.4)' )  &
            ifnrawi(1:nc),iyear,'-',imonth,'-',idate,'-',ihour
   open(10,file=locfn,status='unknown',form='formatted')
   rewind 10
   call read_rawi(10  &
        ,obslat1,obslat2,obslon1,obslon2)
   close(10)
endif

if( isurface.eq. 1) then
!-----------------------------------------------------------------
!  Now process the surface obs in a similar manner to the rawindsondes.
!    Open a file for the observations with
!    the timestamp. Then call a routine to read the obs,
!    do some optional processing, then write them to this file.
!    A sample routine is found in "read_sfc.f".
!-----------------------------------------------------------------
   nc=len_trim(ifnsfc)
   write(locfn,'(a,i4.4,a1,i2.2,a1,i2.2,a1,i4.4)' ) &
        ifnsfc(1:nc),iyear,'-',imonth,'-',idate,'-',ihour
   open(10,file=locfn,status='unknown',form='formatted')
   rewind 10
   call read_sfc(10  &
        ,obslat1,obslat2,obslon1,obslon2)
   close(10)
endif

end
!
!     ***************************************************************
!
SUBROUTINE PRESSIO(IUN,IFN,D,n1,n2,n3)

implicit none

integer :: iun,n1,n2,n3
real :: d(n1,n2,n3,*)
character(len=*) ifn

include 'prep_com.h'

integer :: inproj,ind2,itinc
integer :: lv,i,j,l,n
real :: reflat1,reflon,reflat2,elon,tnlat

PRINT 10, IUN,IFN
10   FORMAT(//,'Writing pressure file on unit ',I2,' .',/  &
     ,'  Filename - ',A40)

elon = wplon+(nprx-1)*spcnprx
tnlat= splat+(npry-1)*spcnpry

! beg ralph2 header for gridded lat-lon data
          write(iun,*)999999,2
          inproj = 1
          reflat1 = 0.
          reflon = 0.
          reflat2 = 0.
          ind2 = 0
          write(iun,900)iyear,imonth,idate,ihour,itinc &
            ,nprz,nprx,npry &
            ,inproj,spcnprx,spcnpry,splat,wplon,tnlat,elon &
            ,reflat1,reflon,reflat2,ind2,(levpr(l),l=1,nprz)
 900      format(8i6,/,i1,2x,9(f10.4,2x),/,i1,(19i6))
! end ralph2 header

do lv=1,nprz
   do n=1,5
      write(iun,950) ((d(i,j,lv,n),i=1,nprx),j=1,npry)
950        format(8f10.3)
   enddo
enddo

close(iun)

PRINT 102,' *****  Pressure file written *************'  &
     ,iyear,imonth,idate,ihour,wplon,elon,splat,tnlat  &
     ,spcnprx,spcnpry,nprx,npry,nprz,(levpr(l),l=1,nprz)
102  format(////,a45,//  &
     ,7X,' Date (year,month,day,hour)  - ',4I5,/  &
     ,7X,' Lon-lat bounds              - ',4F8.2,/  &
     ,7X,' Grid spacing (degrees)      - ',2F8.2,/  &
     ,7X,' Number of X,Y points        - ',2I5,/  &
     ,7X,' Number of pressure levels   - ',I5,/  &
     ,7X,' Pressure levels  (mb)       - '/,(32X,8I5))


RETURN
END
